/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <p18f26j50.h>            /* HiTech General Includes */
#include <flash.h>


#include "user.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/
void StartI2C(void){
	Delay();
	SDA_TRIS = 0;
	Delay();
	SCL_TRIS = 0;
	Delay();
}
void StopI2C(void){
	Delay();
	SDA_TRIS = 0;
	Delay();
	SCL_TRIS = 1;
	Delay();
	SDA_TRIS = 1;
	Delay();	
}
char WriteByte(short data){
	int i = 8,ack;
	while(i--){
		Delay();
		Delay();
		SDA_TRIS = (data & 0x80) ? 1 : 0;
		data <<= 1;
		Delay();
		Delay();
		SCL_TRIS = 1;
		Delay();
		Delay();
		SCL_TRIS = 0;
		if(i==0) break;
	}

//-------READ ACK---------
	SDA_TRIS = 1;
	Delay();
	Delay();
	SCL_TRIS = 1;
	Delay();
	Delay();
	ack = SDA_PORT;	
	SCL_TRIS = 0;
	Delay();
	Delay();

	return ack;

}
char SendCommand(char address, char command){
	char ack;

	StartI2C();
	Delay();
	ack = WriteByte(address);
	Delay();
	StopI2C();

	StartI2C();
	Delay();
	WriteByte(command);
	Delay();
	StopI2C();
	Delay();

	
	return ack;
}
void Delay(void){
	unsigned int i = 300;
	while(i--){
		if(i == 1) break;
	}
}
void UART(void){
	unsigned int i = 50;
	TRISCbits.TRISC6 = 0;
	while(i){
		LATCbits.LATC6 = 1;
		Delay();
		i--;
		if(i==0) break;
	}
}
void ScanAddresses(){
	char i = 0xF0, ack, j = 0;
	Erase_Flash();
	SendCommand(0x00,0x00);
	while(j < 0x5){
		ack = SendCommand(i,0x00);		
		if(!ack)WriteWordFlash(0x6010 + 2*(j),i);
		j++;
		i++;
	}
}
char FirstFree(){
	unsigned short *data;
	char addr = 0xF0;
	while(addr < 0xF5){
		ReadFlash(0x6010 + 2*(addr-0xF0),2,data);
		if(*data == 0xFFFF) break;
		addr++;
	}
	WriteWordFlash(0x6050,addr);
	return addr;
}
void Erase_Flash(){
	DWORD_VAL  addr;

	addr.Val = 0x6000 ;
	TBLPTRU = addr.byte.UB;
	TBLPTRH = addr.byte.HB;
	TBLPTRL = addr.byte.LB;

	EECON1bits.WREN = 1;
	EECON1bits.FREE = 1;
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0x0AA;
	EECON1bits.WR = 1;
	INTCONbits.GIE = 1;
}
void InitApp(void)
{
    
	//BUTTON
	TRISBbits.TRISB0 = 1;
	INTCON2bits.RBPU = 0;
	ANCON1bits.PCFG12 = 1;

	//I/O Pins
	TRISA &= (~0x2F);
	LATA |= 0x2f;
	TRISC &= (~0x07);
	LATC |= 0x07;

	SDA_TRIS = 1;
	SCL_TRIS = 1;
	SDA_LAT = 0;
	SCL_LAT = 0;

}

